#include "motore2d.h"

typedef struct PAL{
	char r;
        char g;
        char b;
}TYPE_PALETTE;

int X_SIZE=320; //LARGHEZZA DELLO SCHERMO
int Y_SIZE=200; //ALTEZZA DELLO SCHERMO
unsigned int SCREEN_SIZE=X_SIZE*Y_SIZE; //GRANDEZZA DELLO SCHERMO

char FIRE_HEIGHT = 1;          /* Step per la sfumatura verso il nero*/
unsigned char MAX_HEIGHT=128;  //Y_SIZE / 100;


TYPE_PALETTE firepal[256];     /* La palette del fuoco */

void generate_fire_palette (TYPE_PALETTE *pal)
{
 unsigned char n;

 /* Il primo colore lo settiamo per default al nero */
 pal[0].r = 0;      pal[0].g = 0;       pal[0].b = 0;

 /* L'ultimo colore lo settiamo per default al bianco */
 pal[255].r = 63;   pal[255].g = 63;    pal[255].b = 63;

 /* I primi 128 colori sono organizzati secondo una sfumatura graduale dal
    nero al rosso vivo */
 for (n=1; n<128; n++)
  {
   pal[n].r = n/2;
   pal[n].g = 0;
   pal[n].b = 0;
  }

 /* Altri 64 colori li utilizziamo per le sfumature di giallo della fiamma */
 for (n=128; n<192; n++)
  {
   pal[n].r = 63;
   pal[n].g = n-128;
   pal[n].b = 0;
  }

 /* Gli ultimi 64 colori a nostra disposizione li utilizziamo per realizzare
    una sfumatura dal giallo al bianco vivo */
 for (n=192; n<255; n++)
  {
   pal[n].r = 63;
   pal[n].g = 63;
   pal[n].b = n-192;
  }
}

/*-------------------------------------------------------------------------*/
/* FUNZIONE move_fire                                                      */
/* UTILIZZO Crea il fuoco dal buffer e lo "muove" verso l'alto             */
/* INPUT    Nessuno                                                        */
/* OUTPUT   Nessuno                                                        */
/*-------------------------------------------------------------------------*/
void move_fire (void)
{
 unsigned int x, y;

  for (x=0; x<SCREEN_SIZE; x++)
  {
   virtualscreen[x] = (virtualscreen[x+1] + virtualscreen[x+X_SIZE-1] + virtualscreen[x+X_SIZE] + virtualscreen[x+X_SIZE+1])>>2;
   if ((virtualscreen[x] < MAX_HEIGHT)&&(virtualscreen[x]>0)) virtualscreen[x] -=FIRE_HEIGHT;
   else if(virtualscreen[x] <220 )virtualscreen[x]+=FIRE_HEIGHT;
  }
}
/*---------------------------------------------------------------------------*/
/* FUNZIONE update_last_lines                                                */
/* UTILIZZO Aggiorna le ultime 2 linee del buffer virtuale con valori casuali*/
/* INPUT    Nessuno                                                          */
/* OUTPUT   Nessuno                                                          */
/*---------------------------------------------------------------------------*/
void update_last_lines (void)
{
 int x;
 for (x=0; x<X_SIZE; x++)
   {
    virtualscreen[SCREEN_SIZE+x] = (rand () % 256);
    virtualscreen[SCREEN_SIZE+X_SIZE+x] = (rand () % 100)+1;
   }
}

void set_palette(TYPE_PALETTE *pal_buf)
{
int	i;
	for(i=0;i<256;i++)
	{
		outp(PALETTE_MASK,0xff);
		outp(PALETTE_REGISTER_WR,i);
		outp(PALETTE_DATA,pal_buf[i].r);
		outp(PALETTE_DATA,pal_buf[i].g);
		outp(PALETTE_DATA,pal_buf[i].b);
	}
}
/*-------------------------MAIN--------------------------*/

main()
{
printf("Routine del fuoco creata da cattai Marco il 30-03-1998\n");
printf("*****************************************************\n ");
printf("                   PREMI UN TASTO     		        ");
getch();
if(create_screen(1)==1)return 1;
set_vga_mode  ();
 clrscreen(virtualscreen);
 generate_fire_palette (firepal);   /* Creiamo un'opportuna palette */
 set_palette (firepal);             /* Settiamola */
 while (!kbhit())                   /* Attendiamo la pressione di un tasto */
  {
   update_last_lines ();
   move_fire ();                    /* Ciclo principale del fuoco */
   copyscr(physicalscreen,virtualscreen);
  }
 farfree(virtualscreen);
 farfree(tempscreen);
set_text_mode ();
return 0;
}